/* 
   Rombank (Java Edition) - A imbNES rombank.bin generator for multiple platforms
   Author: Bruno Freitas (aka Sparrow) - 03/2004
   Contact: bootsector@ig.com.br
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

import java.io.*;
import java.util.*;

/** This class encapsulates a directory filled of NES ROMs and provides methods for
 * handling these ROMs.
 */
public class RomsList {
   private File directory;
   private File roms[];
   
   /** Class constructor. It receives the full path for the directory containing the
    * NES roms.
    * @param dir Directory that contains the ROMS.
    */   
   public RomsList(String dir) {
      directory = new File(dir);
      
      FileFilter filter = new FileFilter() {
         public boolean accept(File file) { 
            return (file.isFile() && file.getName().toUpperCase().endsWith(".NES"));
         } 
      }; 

      Comparator order = new Comparator() {
         public int compare(Object a, Object b) {
            File filea = (File)a;
            File fileb = (File)b;

            return filea.getName().compareToIgnoreCase(fileb.getName());
         }
      };
      
      if (!directory.isDirectory()) return;
      
      roms = directory.listFiles(filter);
      
      Arrays.sort(roms, order);
   }
   
   /** Returns the size (in bytes) of the ROM pointed by index.
    * @param index The ROM index. It can be a value from 0 to romsCount()-1.
    * @return Returns the ROM size, in bytes.
    */   
   public long romSize(int index) {
      if (index < 0 || index >= romsCount())
         return -1;
      
      return roms[index].length();
   }
   
   /** Returns the file name (plus extension, without the path) of the ROM pointed by
    * index.
    * @param index The ROM index. It can be a value from 0 to romsCount()-1.
    * @return Returns the ROM's file name.
    */   
   public String romFileName(int index) {
      if (index < 0 || index >= romsCount())
         return "";

      return roms[index].getName();
   }
   
   /** Returns the full file name (plus path and extension) of the ROM pointed by
    * index.
    * @param index The ROM index. It can be a value from 0 to romsCount()-1.
    * @return Returns the ROM's full file name.
    */   
   public String romFullFileName(int index) {
      if (index < 0 || index >= romsCount())
         return "";

      return roms[index].getAbsoluteFile().toString();
   }
   
   /** Returns the roms count in the list.
    * @return Returns the roms count in the list.
    */   
   public int romsCount() {
      if (roms == null) return 0;
      else return roms.length;
   }
   
   /** This methos reads the ROM's content to the array data[]. Note that you must
    * allocate sufficient space to data[] before calling this method.
    *
    * An example could be something like: byte romContents[] = new byte[(int)romsList.romSize(i)];
    * @param index The ROM index. It can be a value from 0 to romsCount()-1.
    * @param data The array which will receive the ROM's data. It must be allocated before calling
    * this method.
    */   
   public void romReadData(int index, byte data[]) {
      if (index < 0 || index >= romsCount())
         return;

      try {
         FileInputStream fileinput = new FileInputStream(roms[index]);
         fileinput.read(data);
         fileinput.close();
      } catch (Exception e) {
         System.err.println("Error opening file: " + romFullFileName(index));
      }
   }
}
