/* 
   Rombank (Java Edition) - A imbNES rombank.bin generator for multiple platforms
   Author: Bruno Freitas (aka Sparrow) - 03/2004
   Contact: bootsector@ig.com.br
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

import java.io.*;

/** This class implements the rombank Table Of Contents, which, according to Allan
 * Blomquist is something like this:
 *
 * "The rombank format is really simple. the first part is a TOC with each
 * game having a 32 byte entry. the first 28 bytes are the title of the
 * game and the last 4 bytes are a 32 bit, little endian word, the lower
 * 20 bits of which are the sector offset that the game starts at and the
 * upper 12 bits are the size of the game in sectors (a sector is 0x800
 * bytes). terminate the TOC with an extra entry with the first byte of
 * the title set to 255. then starting with the next sector after the TOC,
 * you just put all the games. that's about all there is too it.
 * i'd recommend making a simple rombank.bin with a few games and just
 * looking at it with a hex editor. i think everything i wrote here (off
 * the top of my head) should match up..."
 */
public class TOC {
   final int sectorSize = 0x800; // 2048 decimal
   
   private RomsList romsList;   
   private int SizePos[];

   /** Class constructor. It requires a RomList object as parameter.
    * @param roms A RomList object which will provide all data for TOC generating.
    */   
   public TOC(RomsList roms) {
      romsList = roms;
      SizePos = new int[roms.romsCount()];
      fillSizePos();
   }
   
   /** This method returns the Menu Name for every game. It currently support the
    * following TAG list:".nes;[*]", just link rombank for Windows. It's not possible
    * for the user modify that TAG removal list without mess around with the source
    * code, yet! ;-)
    * @param index The index of the game to which the Menu Name will be retrieved.
    * @return The returned value is a string of 28 byte maximum.
    */   
   public String menuName(int index) {
      if (index < 0 || index >= romsList.romsCount())
         return "                            ";
      
      String mnName = romsList.romFileName(index);

      // TAG removal list. Defaults to: .nes;[*]
      // to-do: Let the user choose the TAG removal list
      mnName = mnName.replaceAll("\\[.*\\]","");
      mnName = mnName.replaceAll("\\.[Nn][Ee][Ss]","");
      mnName = mnName.trim();
      
      // mnName plus 28 spaces
      mnName = mnName + "                            ";
      
      mnName = mnName.substring(0, 28);
      
      return mnName;
   }
   
   /** This method returns the size of the rom in sectors and its offset in the
    * rombank. Both data are encoded in an Integer value.
    * @param index The index of the ROM.
    * @return This method returns an Integer which contains both the ROM size (in sectors) and
    * it offset in the rombank.
    */   
   public int getSizePos(int index) {
      if (index < 0 || index >= romsList.romsCount())
         return 0;
      
      return SizePos[index];
   }

   private void fillSizePos() {
      int startPos = (int)Math.ceil((double)romsList.romsCount() * 32 / sectorSize);
      int romSize; // in Sectors
      
      for (int i = 0; i < SizePos.length; i++) {
         romSize = (int)Math.ceil((double)romsList.romSize(i) / sectorSize);
         SizePos[i] = (romSize << 20) | startPos;
         startPos += romSize;
      }
   }
   
   /** This method write the TOC to the rombank file.
    * @return This method returns false if something unexpected occur!!
    * @param filename The name of the file on the disk to write down the TOC.
    */   
   public boolean writeTOC(String filename) {
      if (romsList.romsCount() < 1) {
         System.out.println("\nError! No ROMs where found!");
         return false;
      }
      
      // Check for Max Roms Limit (4095)!!
      if (romsList.romsCount() > 4095) {
         System.out.println("\nError! Too much ROMs found! You can use a maximum of 4095 ROMs!");
         return false;
      }

      try {
         RandomAccessFile outfile = new RandomAccessFile(filename, "rw");
         
         outfile.setLength(0);
         
         for(int i = 0; i < SizePos.length; i++) {
            outfile.write(menuName(i).getBytes());
            outfile.writeInt(toLittleEndian(getSizePos(i)));
         }
         
         // Write the ending TOC entry...
         outfile.write(0xFF);
         for(int i = 1; i <= 27; i++) outfile.write(0x20);
         outfile.writeInt(toLittleEndian(getSizePos(romsList.romsCount()-1)));
         
         // Pad the sector...
         // TOCSize - In bytes;
         int TOCSize = (int)Math.ceil((double)romsList.romsCount() * 32 / sectorSize) * sectorSize;
         int padSize = TOCSize - ((romsList.romsCount() + 1) * 32); // Can't forget ending TOC entry!
         
         for(int i = 1; i <= padSize; i++)
            outfile.write(0x00);
         
         // Closes file
         outfile.close();
      } catch (Exception e) {
         System.err.println("An error has occurred while saving file " + filename);
         return false;
      }
      
      return true;
   }
   
   private int toLittleEndian(int v) {
      return (v >>> 24) | (v << 24) | ((v << 8) & 0x00FF0000) | ((v >> 8) & 0x0000FF00);
   }
}
